<!DOCTYPE html>
<html>
    <head>
        <meta http-equiv="Content-type" content="text/html" charset="UTF-8" />
        <meta name="viewport" content="width=device-width" initial-scale="1" />
        <title>{{title}}!</title>
        <link rel="stylesheet" href="{{asset:stylish.css}}" />
        {{bearlib}}
    </head>
    <body>
        {{dev}}
        {{body}}
        <h3>Box</h3>
        <p>The background should be pink.</p>
        <div id="box">
            <p>Dear {{name}}, this is the box.</p>
        </div>
    </body>
</html>
