<!DOCTYPE html>
<html>
    <head>
        <title>{{title}}</title>
        <link rel="stylesheet" href="{{asset:index-style.css}}" />
    </head>
    <body>
        {{dev}}
        <div id="content">
            {{body}}
        </div>
    </body>
</html>
