<!DOCTYPE html>
<html>
    <head>
        <meta http-equiv="Content-type" content="text/html" charset="UTF-8" />
        <meta name="viewport" content="width=device-width" initial-scale="1" />
        <title>{{title}}</title>
        {{resources}}
        {{bearlib}}
    </head>
    <body>
        {{dev}}
        {{body}}
        {{extra}}
    </body>
</html>
