body {
    margin: auto;
    width: 1000px;
    font-size: 16px;
}
table {
    border-collapse: collapse;
}
td {
    padding: 5px;
    vertical-align: top;
}
td.url {
    color: white;
    background: darkblue;
    min-width: 50px;
}
td.url > a:hover {
    color: cyan;
}
td.url > a {
    display: inline-block;
    color: white;
    width: 100%;
    height: 100%;
}
